"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Driver = void 0;
const sharp_1 = __importDefault(require("sharp"));
const utils_1 = require("./utils");
const bent_1 = __importDefault(require("bent"));
class Driver {
    constructor({ name }) {
        this.name = name;
        return new Proxy(this, {
            get: (target, prop) => {
                if (typeof target[prop] !== 'function' || target[prop].name === 'url') {
                    return target[prop];
                }
                return (...args) => __awaiter(this, void 0, void 0, function* () {
                    try {
                        return yield target[prop](...args);
                    }
                    catch (error) {
                        if (!this.errorHandler) {
                            throw error;
                        }
                        this.errorHandler(error);
                    }
                });
            },
        });
    }
    /**
     * Upload image from specific URI and store it into `filename`
     *
     * @param uri URI of image
     * @param path Filename included location to store image.
     * @param ignoreHeaderContentType ignore checking content-type header.
     * @returns Promise<any>
     */
    uploadImageFromExternalUri(uri, path, ignoreHeaderContentType = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const head = bent_1.default('HEAD');
            const get = bent_1.default('GET', 200, 'buffer');
            const res = yield head(uri);
            if (ignoreHeaderContentType ||
                (res.headers['content-type'] && res.headers['content-type'].includes('image'))) {
                const imageBuffer = yield get(uri);
                return this.put(imageBuffer, path);
            }
            throw new Error('Not an image: ' + uri);
        });
    }
    imageStats(path, keepBuffer = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const stream = yield this.get(path);
            const buffer = yield utils_1.streamToBuffer(stream);
            const { format, size, width, height } = yield sharp_1.default(buffer).metadata();
            return Object.assign({ name: utils_1.getFileName(path), path, size: utils_1.bytesToKbytes(size), width,
                height, mime: format, ext: utils_1.getExt(path), hash: null }, (keepBuffer && { buffer }));
        });
    }
}
exports.Driver = Driver;
