"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.bytesToKbytes = exports.getFileName = exports.getExt = exports.streamToBuffer = exports.getRootCwd = exports.requireDefaultModule = exports.exists = exports.toStream = exports.ensureDirectoryExistence = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const stream_1 = require("stream");
const ensureDirectoryExistence = (filePath) => {
    const dirname = path_1.default.dirname(filePath);
    if (fs_1.default.existsSync(dirname)) {
        return true;
    }
    fs_1.default.mkdirSync(dirname, { recursive: true });
};
exports.ensureDirectoryExistence = ensureDirectoryExistence;
/**
 * Convert data to readable stream if it is a buffer or string.
 */
// TODO rename to toReadable.
function toStream(buf, chunkSize) {
    if (typeof buf === 'string') {
        buf = Buffer.from(buf, 'utf8');
    }
    if (!Buffer.isBuffer(buf)) {
        if (!(buf instanceof stream_1.Readable)) {
            const pass = new stream_1.PassThrough();
            buf.pipe(pass);
            return pass;
        }
        return buf;
    }
    const reader = new stream_1.Readable();
    const hwm = reader.readableHighWaterMark;
    // If chunkSize is invalid, set to highWaterMark.
    if (!chunkSize || typeof chunkSize !== 'number' || chunkSize < 1 || chunkSize > hwm) {
        chunkSize = hwm;
    }
    const len = buf.length;
    let start = 0;
    // Overwrite _read method to push data from buffer.
    reader._read = function () {
        while (reader.push(buf.slice(start, (start += chunkSize)))) {
            // If all data pushed, just break the loop.
            if (start >= len) {
                reader.push(null);
                break;
            }
        }
    };
    return reader;
}
exports.toStream = toStream;
/**
 * Check if file/directory exists in local
 */
function exists(path) {
    return new Promise((resolve, reject) => {
        fs_1.default.stat(path, (err) => {
            if (err) {
                reject(err);
                return;
            }
            resolve();
        });
    });
}
exports.exists = exists;
/**
 * Require a module es module as its default value.
 */
function requireDefaultModule(path, notThrow = true) {
    try {
        return require(path).default;
    }
    catch (error) {
        if (notThrow) {
            return null;
        }
        throw error;
    }
}
exports.requireDefaultModule = requireDefaultModule;
/**
 * Get root directory.
 */
function getRootCwd() {
    const cwd = process.cwd();
    const cwdArr = cwd.split('/');
    if (cwdArr[cwdArr.length - 2] === 'packages') {
        cwdArr.splice(cwdArr.length - 2, 2);
        return cwdArr.join('/');
    }
    return cwd;
}
exports.getRootCwd = getRootCwd;
function streamToBuffer(stream) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            const _buf = Array();
            stream.on('data', (chunk) => {
                _buf.push(chunk);
            });
            stream.on('end', () => {
                resolve(Buffer.concat(_buf));
            });
            stream.on('error', (err) => reject(`error converting stream - ${err}`));
        });
    });
}
exports.streamToBuffer = streamToBuffer;
function getExt(filepath) {
    return filepath.split('?')[0].split('#')[0].split('.').pop();
}
exports.getExt = getExt;
/**
 * Get file name from give path.
 */
function getFileName(filePath) {
    return filePath
        .split('?')[0]
        .split('#')[0]
        .replace(/^.*[\\/]/, '');
}
exports.getFileName = getFileName;
const bytesToKbytes = (bytes) => Math.round((bytes / 1000) * 100) / 100;
exports.bytesToKbytes = bytesToKbytes;
